/*
 * Example1.java
 *
 * Umgestellt auf lokales LgDatum statt java.sql.Date
 * Created on 28. Februar 2001, 15:50
 */

package bank3tier._lgTest;

import java.util.List;

import bank3tier.lg.LgSitzung;
import bank3tier.lg.LgDatum;
import bank3tier.lg.LgKonto;
import bank3tier.lg.LgKunde;
import bank3tier.lg.LgKontozugriff;

/**
  Zweck: Erzeugt einige Kunden sowie einige von diesen verwaltete Konten
  und führt einige Überweisungen aus.

  @author  Christoph Knabe
 */
public class Example1 extends Object {

    private LgSitzung _lgApplication;
    
    /**Do some work using the application logic tier (Lg) */
    public static void main (final String[] i_args){
        final Example1 example = new Example1();
        try{example.work();}
        catch(Exception ex){multex.Msg.printReport(ex);  System.exit(1);}
    }

    /**create application object and and empty database, then do modifications.*/
    public void work() throws Exception {
        _lgApplication = new LgSitzung(); 
        
        doModifications();
    }//work

    /**Does the example modifications to the database as one transaction, either completely or nothing at all.*/
	private void doModifications() throws Exception {
        _lgApplication.beginTransaction();
        boolean ok = false;
		try {
			//Einige Kunden erfassen:
			final LgKunde mueller 
            = _lgApplication.kundeErfassen("Manfred Müller", LgDatum.fromString("27.02.1960"));

            final LgKunde knabe 
            = _lgApplication.kundeErfassen("Christoph Knabe", LgDatum.fromString("31.05.1954"));

            final LgKunde skoric 
            = _lgApplication.kundeErfassen("Gordana Skoric-Knabe", LgDatum.fromString("14.11.1962"));

			//Für diese Kunden Konten eröffnen:
            final LgKonto muellerGiroKonto = mueller.kontoEroeffnen("muellerGiroKonto").getKonto();
            final LgKonto muellerSparKonto = mueller.kontoEroeffnen("muellerSparKonto").getKonto();
            final LgKonto knabeKonto = knabe.kontoEroeffnen("knabeKonto").getKonto();
            final LgKonto guthsmuthsVereinskonto = knabe.kontoEroeffnen("guthsmuthsVereinskonto").getKonto();

            //Skoric als Verwalter für knabeKonto einsetzen:
            knabe.verwalterHinzufuegen(knabeKonto, skoric);
            //Müller als Verwalter für Konto des TSV Guthsmuths einsetzen:
            knabe.verwalterHinzufuegen(guthsmuthsVereinskonto, mueller);

			//Einige Überweisungen tätigen:
            muellerGiroKonto.ueberweisen(muellerSparKonto, 200000, mueller);
            muellerGiroKonto.ueberweisen(guthsmuthsVereinskonto, 2000, mueller);
            knabeKonto.ueberweisen(muellerGiroKonto, 1998, knabe);

			//Zeige Liste aller Kunden:
            final List<LgKunde> kundeList = this._lgApplication.alleKundenSuchen();
			System.out.println("Liste aller Kunden");
			System.out.println("==================");
			System.out.println("oid,\tName,\tGeburt");
			for(final LgKunde k: kundeList){
			  System.out.println(k.getOid() + ",\t" + k.getName() + ",\t" + k.getGeburt()); 
			}
			//Zeige Status und Kontostand aller von Kunde 0 verwalteten Konten:
			final LgKunde kunde0 = kundeList.get(0);
			kunde0.printKontenStaende();

			//Demonstriert eindeutiges Suchen:
			final LgDatum suchDatum = LgDatum.fromString("31.05.1954");
			final LgKunde geb540531Kunde = _lgApplication.kundenSuchen(suchDatum);
            System.out.println("Zeige den Kunden, der am " + suchDatum + " geboren wurde:");
			System.out.println(geb540531Kunde);
			
		}finally{_lgApplication.endTransaction(ok);}
	}

    /** Do not create objects of me from outside*/
    private Example1() {;
    }//Example1

}//Example1