package bank3tier.db;

import java.util.List;

/**Represents a package-local query using Hibernate as persistence mechanism.
 * Converts the polymorphic Hibernate result List to a typed List with element type T.
 * It does not contain any public methods other than the default constructor.
 * In the logic layer only the name of its subclasses are to be used in the form
 * MyQuery.class in order to distinguish this query.
 * @author Christoph Knabe 2005-05-07
 **/
public abstract class HibernateQuery {
    
    /**Executes this Hibernate query.
     * @param io_hibernatePool The pool which will be queried. 
     * @param i_resultClass The class, whose objects are searched.
     * @param i_args Arguments for this query.
     * @return A List of objects of the specified result class, which are found by this query.
     */
    /*package*/ abstract <T> List<T> execute(
        final _HibernatePool io_hibernatePool,
        final Class<T> i_resultClass, 
        final Object[] i_args
    ) throws Exception;

}
