package bank3tier.lg;

import java.sql.Date;
import java.util.List;
import java.util.Set;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import multex.Failure;

import bank3tier.db.DbJungeKundenQuery;
import bank3tier.db.Pool;
import bank3tier.db.Pool.ArgumentCountExc;
import bank3tier.db.Pool.ArgumentTypeExc;
import bank3tier.db.Pool.NoPersistentClassExc;
import bank3tier.db.Pool.NoQueryClassExc;
import bank3tier.lg.LgKonto.BetragExc;
import bank3tier.lg.LgKonto.MindestSaldoExc;
import bank3tier.lg.LgKonto.OhneRechteExc;
import bank3tier.lg.LgKunde.KundeHatNochKontenExc;
import bank3tier.lg.LgKunde.SchonVerwalterExc;

public class TestLgApplication extends Assert {

    private static final LgSitzung _lgApplication = new LgSitzung();
    
    /**Erwartete Inhalte der Datenbank*/
    private static LgKunde _mueller, _knabe, _skoric;

    /**Erwartete Inhalte der Datenbank*/
    private static LgKonto _muellerGiroKonto, _muellerSparKonto, _knabeKonto, _guthsmuthsVereinsKonto;
    

    @Before public void setUp(){
        _lgApplication.beginTransaction();
    }
    
    @After public void tearDown(){
        try {
            _lgApplication.endTransaction(true);
        } catch (RuntimeException e) {
            System.err.println("Failure during " + getClass().getName() + ".tearDown():");
            multex.Msg.printReport(e);
        }
    }
    
    @Test public void kundenErfassen(){
        System.out.println("kundenErfassen:");
        //3 Musterkunden erfassen:
        _mueller = _erfassen("Manfred Mueller", "27.02.1960");
        _knabe = _erfassen("Christoph Knabe", "31.05.1954");
        _skoric = _erfassen("Gordana Skoric-Knabe", "14.11.1962");
        assertEquals(new Integer(2), _knabe.getOid());
        assertEquals("Christoph Knabe", _knabe.getName());
        assertEquals("31.05.1954", _knabe.getGeburt().toString());
    }

    @Test public void einzelnenSuchen()
    throws Exception
    {
        System.out.println("einzelnenSuchen");
        final LgKunde skoric
        = _lgApplication.kundenSuchen(LgDatum.fromString("14.11.1962"));
        assertEquals(_skoric.toString(), skoric.toString());
        
        final LgKunde knabe = _lgApplication.getPool().find(LgKunde.class, new Integer(2));
        assertEquals(_knabe.toString(), knabe.toString());

        try{
            _lgApplication.kundenSuchen(LgDatum.fromString("01.01.1970"));
        	fail("Kein Kunde gefunden erwartet");
        } catch ( RuntimeException expected){}
    }

    @Test public void alleSuchen(){
        System.out.println("alleSuchen");
        final List<LgKunde> kunden = _lgApplication.alleKundenSuchen();
		assertEquals(3, kunden.size());

        final LgKunde skoric = kunden.get(0);
        assertEquals(_skoric.toString(), skoric.toString());  

        final LgKunde knabe   = kunden.get(1);
        assertEquals(_knabe.toString(), knabe.toString());

        final LgKunde mueller  = kunden.get(2);      
        assertEquals(_mueller.toString(), mueller.toString());

    }

    @Test public void aendern()
    throws Exception
    {
        
        final String knabeGeburt = "31.05.1954";
		final LgKunde knabeAlt
        = _lgApplication.kundenSuchen(LgDatum.fromString(knabeGeburt));
        assertEquals(_knabe.toString(), knabeAlt.toString());

        knabeAlt.setName("Prof. Knabe");
        final String geaenderteGeburt = "31.12.2000";
		knabeAlt.setGeburt(LgDatum.fromString(geaenderteGeburt));
        knabeAlt.aendern();
		_knabe = knabeAlt;

        final LgKunde knabeNeu
        = _lgApplication.kundenSuchen(LgDatum.fromString(geaenderteGeburt));
        assertEquals(_knabe.toString(), knabeNeu.toString());
        
        //Sollzustand wiederherstellen:
        knabeNeu.setGeburt(LgDatum.fromString(knabeGeburt));
        knabeNeu.aendern();

        final LgKunde knabeNeuNeu
        = _lgApplication.kundenSuchen(LgDatum.fromString(knabeGeburt));
        assertEquals(knabeNeu.toString(), knabeNeuNeu.toString());
    }

    @Test public void jungeKundenSuchen() throws Exception {
        final List<LgKunde> kunden = _lgApplication.jungeKundenSuchen(LgDatum.fromString("01.01.1960"));
        assertEquals(2, kunden.size());

        final LgKunde mueller = kunden.get(0);
        assertEquals(_mueller.toString(), mueller.toString());

        final LgKunde skoric  = kunden.get(1);
        assertEquals(_skoric.toString(), skoric.toString());
    }

    @Test public void kontoEroeffnen() throws Exception {
        final LgKontozugriff knabeVerwaltet  = _knabe.kontoEroeffnen("_knabeKonto");
        _knabeKonto = knabeVerwaltet.getKonto();
        assertTrue(knabeVerwaltet.isInhaber());
        assertEquals(0, _knabeKonto.getSaldo());
        assertSame(_knabe, knabeVerwaltet.getKunde());
        
        final LgKontozugriff guthsmuthsVereinsVerwaltet = _knabe.kontoEroeffnen("_guthsmuthsVereinsKonto");
        assertTrue(guthsmuthsVereinsVerwaltet.isInhaber());
        _guthsmuthsVereinsKonto = guthsmuthsVereinsVerwaltet.getKonto();
        _muellerGiroKonto = _mueller.kontoEroeffnen("_muellerGiroKonto").getKonto();
        _muellerSparKonto = _mueller.kontoEroeffnen("_muellerSparKonto").getKonto();
    }

    /**Testet die rollenbasierte Methode LgKunde.verwalterHinzufuegen*/
    @Test public void verwalterHinzufuegen() throws Exception {
        //Skoric als Verwalter fuer knabeKonto einsetzen:
        final LgKontozugriff skoricVerwaltet  = _knabe.verwalterHinzufuegen(_knabeKonto, _skoric);
        assertFalse(skoricVerwaltet.isInhaber());
        assertSame(_knabeKonto, skoricVerwaltet.getKonto());
        assertSame(_skoric, skoricVerwaltet.getKunde());

        try {
            _knabe.verwalterHinzufuegen(_knabeKonto, _skoric);
            fail("SchonVerwalterExc expected");
        } catch ( SchonVerwalterExc expected ) {
        }
        try {
            _knabe.verwalterHinzufuegen(_muellerGiroKonto, _skoric);
            fail("OhneRechteExc expected");
        } catch ( OhneRechteExc expected ) {
        }
        

        //Mueller als Verwalter fuer Konto des TSV Guthsmuths einsetzen:
        _knabe.verwalterHinzufuegen(_guthsmuthsVereinsKonto, _mueller);
    }

    @Test public void ueberweisenErfolg() throws Exception {
        final int betrag1 = 200000;
        _muellerGiroKonto.ueberweisen(_muellerSparKonto, betrag1, _mueller);
        assertEquals(betrag1, _muellerSparKonto.getSaldo());
        assertEquals(-betrag1, _muellerGiroKonto.getSaldo());
        _muellerGiroKonto.ueberweisen(_guthsmuthsVereinsKonto, 2000, _mueller);
        _knabeKonto.ueberweisen(_muellerGiroKonto, 1998, _knabe);
    }
    
    /**Testet die negativen Geschaeftsregeln beim Ueberweisen.*/
    @Test public void ueberweisenVerbote() throws Exception {
        try {
            _knabeKonto.ueberweisen(_muellerGiroKonto, 1998, _mueller);
            fail("OhneRechteExc expected");
        } catch ( OhneRechteExc expected ) {
        }
        try {
            _knabeKonto.ueberweisen(_muellerGiroKonto, 0, _knabe);
            fail("BetragExc expected");
        } catch ( BetragExc expected ) {
        }
        try {
            _knabeKonto.ueberweisen(_muellerGiroKonto, -1, _mueller);
            fail("BetragExc expected");
        } catch ( BetragExc expected ) {
        }
        final LgKonto neuesKonto  = _knabe.kontoEroeffnen("neuesKonto").getKonto();
        neuesKonto.ueberweisen(_knabeKonto, -LgKonto.mindestSaldo, _knabe);
        try {
            neuesKonto.ueberweisen(_muellerGiroKonto, 1, _knabe);
            fail("MindestSaldoExc expected");
        } catch ( MindestSaldoExc expected ) {
        }
    }

    @Test public void testPrintGetInhaberKonten() throws Exception {
        System.out.println("testPrintGetInhaberKonten:");
        final List<LgKontozugriff> inhaberKontozugriffe = _lgApplication.printGetInhaberKontozugriffe();
        //Reihenfolge nach fallenden OIDs:
        _check(inhaberKontozugriffe.get(0), "Prof. Knabe", "neuesKonto");
        _check(inhaberKontozugriffe.get(1), "Manfred Mueller", "_muellerSparKonto");
        _check(inhaberKontozugriffe.get(2), "Manfred Mueller", "_muellerGiroKonto");
        _check(inhaberKontozugriffe.get(3), "Prof. Knabe", "_guthsmuthsVereinsKonto");
        _check(inhaberKontozugriffe.get(4), "Prof. Knabe", "_knabeKonto");
        assertEquals(5, inhaberKontozugriffe.size());
    }
    
    private void _check(final LgKontozugriff kontozugriff, final String kundeName, final String kontoBezeichnung) {
        assertTrue(kontozugriff.isInhaber());
        assertEquals(kundeName, kontozugriff.getKunde().getName());
        assertEquals(kontoBezeichnung, kontozugriff.getKonto().getBezeichnung());
    }

    @Test public void reicheKundenSuchen() throws Exception {
        final int reichSaldoCent = 1000*100;
        final List<LgKontozugriff> kzs = _lgApplication.reicheKundenSuchen(reichSaldoCent);
        assertEquals(2, kzs.size());
        //Reihenfolge willkuerlich:
        final LgKunde knabe = kzs.get(0).getKunde();
        assertEquals(_knabe.toString(), knabe.toString());
        
        final LgKunde mueller  = kzs.get(1).getKunde();
        assertEquals(_mueller.toString(), mueller.toString());
    }

    /**Tests universal pool query for exceptions 
     * NoPersistentClassExc, NoQueryClassExc, ArgumentCountExc, ArgumentTypeExc
    */
    @Test public void queryExceptions() throws Exception {
    	final Pool pool = _lgApplication.getPool();
        final Date date = LgDatum.fromString("01.01.1960").toDate();
        try {
            pool.findManyByQuery(LgKunde.class, LgKunde.class, new java.sql.Date[]{date});
            fail("NoQueryClassExc expected");
        } catch ( NoQueryClassExc expected){}
        try {
            pool.findManyByQuery(LgKunde.class, DbJungeKundenQuery.class, new java.sql.Date[]{});
            fail("ArgumentCountExc expected");
        } catch ( ArgumentCountExc expected){}
        try {
            pool.findManyByQuery(LgKunde.class, DbJungeKundenQuery.class, new java.sql.Date[]{date,date});
            fail("ArgumentCountExc expected");
        } catch ( ArgumentCountExc expected){}
        try {
            pool.findManyByQuery(LgKunde.class, DbJungeKundenQuery.class, new Object[]{"hallo"});
            fail("ArgumentTypeExc expected");
        } catch ( ArgumentTypeExc expected){}
    }

    @Test public void kundenLoeschenVerbot() throws Exception {
        try {
            _knabe.loeschen();
            fail("KundeHatNochKontenExc expected");
        } catch (KundeHatNochKontenExc expected) {
        }
    }
    /**Erfasst einen Kunden mit diesem Namen und diesem Geburtsdatum persistent.
     * @param i_geburt Geburtsdatum in lokaler Schreibweise
    */
    private LgKunde _erfassen(final String i_name, final String i_geburt){
        return _lgApplication.kundeErfassen(i_name, LgDatum.fromString(i_geburt));
    }


}//class
